/*
 * biji-tomboy-reader.h
 * 
 * Copyright 2013 Pierre-Yves Luyten <py@luyten.fr>
 * 
 * bijiben is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * bijiben is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#pragma once

#include <glib-object.h>

G_BEGIN_DECLS

#define BIJI_TYPE_TOMBOY_READER (biji_tomboy_reader_get_type ())

G_DECLARE_FINAL_TYPE (BijiTomboyReader, biji_tomboy_reader, BIJI, TOMBOY_READER, GObject)

gboolean             biji_tomboy_reader_read              (gchar *source,
                                                           GError **result,
                                                           BijiInfoSet **set,
                                                           gchar       **html,
                                                           GList **notebooks);


G_END_DECLS
