body {
  max-width:        35.0em;
  margin:           0 auto;
  word-wrap:        break-word;
  color:            black;
  background-size:  2.0em 2.0em;
  line-height:      1.5em;
  padding:          2.0em;
  font-size:        150%;
  font-weight:      bold;
}

ul {
  list-style-position:inside;
  margin-top :      0;
  margin-bottom:    0;
  padding:          0 0 0 .75em;
}

ol {
  list-style-position:inside;
  margin-top :      0;
  margin-bottom:    0;
  padding:          0 0 0 .75em;
}

body > div {
  margin:           0.75em 0;
  font-size:        100%;
  font-weight:      normal;
}
